/****************************************************************/
/*   NOISE GENERATION ROUTINES USING THE			*/
/*   WE(R) DSP16 DIGITAL SIGNAL PROCESSOR			*/
/*   November, 1988 Doc# AP88-18DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1988 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*								*/
/*			errf.c					*/
/*								*/
/* Gaussian or Error function					*/
/* This function claculates the Gaussian/Error function and can	*/
/* be used to calculate cumulative distribution function (CDF)	*/
/* of a Gaussian random variable. The funtion implemented is:	*/
/*                                   x				*/
/*    errf(x) = 1/sqrt(2*PI) * INTGRL e^(-y^2/2) dy		*/
/*                                   0				*/
/* The integral is calculated using the triangle method of	*/
/* integration. The increment used has been experimentally	*/
/* found to be correct to five significant digits.		*/

float errf(x)
float x;
{
	double exp(), asin(), sqrt();
	double fac_twopi;
	double xval, xval0;
	double incr;
	double sum, tmp;
	int i;

	/* get multiplication factor */
	fac_twopi = 4.0 * asin(1.0);
/*	printf("twopi = %f", fac_twopi);  */
	fac_twopi = 1.0 / sqrt(fac_twopi);

	/* set up counter variables */
	incr = x/100	/* this works over a large range of values of x   */
	sum = 0.0;
	xval0 = incr / 2;	/* the mid-point of the intervals	  */

	for ( i=0 ; i<100 ; i++ )
	{
	    xval = xval0 + incr*i;		/* get next mid-point	  */
	    tmp = -1.0 * xval * xval / 2.0;	/* argument for exp	  */
	    tmp = exp(tmp);
	    sum = sum + tmp * incr;		/* sum height times width */
/*	    printf("xval = %f, tmp = %f, sum = %f", xval, tmp, sum);	  */
	}
	sum = sum * fac_twopi;
	return(sum);
}


/* ERRFINV	Inverse of Gaussian or Error Function			*/
/*   This function calculates the inverse of the error function, errf().*/
/*   This is performed by a binary search on the function errf().	*/
/*   The program stops when the passed value x is bounded by 0.001.	*/
/*   YVAL is returned as the inverse of X.				*/
/*   XTST is the true value of X that generates YVAL.			*/

double errfinv(x,y)
double x;
float *y;
{
	float lo, high;
	double xtst;
	float errf();
	int flag;

	lo   = 0;
	high = 10;
	*y   = 5;
	flag = 1;
/*	printf("loop %d, xval = %f, lo, y, hi = %f %f %f",i,xval,lo,y,high); */

	while(flag)
	{
	    xtst = errf(*y);
	    if ( (high - lo) < .001 )
		break;
	    if ( xtst > x )
		high = *y;
	    else
		lo = *y;
	    *y = ( high + lo ) / 2;
/*	    printf("xtst = %f, lo, y, high = %f %f %f",xtst,lo,*y,high); */
	}
	return(xtst);
}
/* end of file */
