/****************************************************************/
/*   NOISE GENERATION ROUTINES USING THE			*/
/*   WE(R) DSP16 DIGITAL SIGNAL PROCESSOR			*/
/*   November, 1988 Doc# AP88-18DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1988 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*								*/
/*			mktab23.c				*/
/*								*/
/* Linear Congruential Method of Random Number Generation.	*/
/* This program generates random numbers based on the Linear	*/
/* Congruential Algorithm,					*/
/*								*/
/*	X(n+1) = ( a * X(n) + c ) mod m				*/
/*								*/
/* The parameters m, a, and c are passed as command line	*/
/* arguments used to generate					*/
/*	Table 2:  appl 134456 8121 28411 2    and		*/
/*	Table 3:  appl 1048576 2045 1 3				*/

main(argc, argv)
int argc;
char *argv[];
{
	long xnp1;	/* x(n+1) */
	long xn;	/* x(n)   */
	long m, a, c;
	int i, tnum;
	float maxval, thisval;

	if(argc < 4)
	{
		printf("Need four arguments, m(l), a(l), c(l), tnum");
		exit();
	}
/*	printf("%s %s %s", argv[1], argv[2], argv[3]);	*/
	sscanf(argv[1]," %ld", &m);
	sscanf(argv[2]," %ld", &a);
	sscanf(argv[3]," %ld", &c);
	sscanf(argv[4]," %l", &tnum);
/*	printf("m = %ld, a = %ld, c = %ld, tnum = %d", m, a, c, tnum);	*/

	/* test for overflow */
	maxval = 1024.*1024.*1024.*2-1;
	thisval = (m-1.)*a + c;
/*	printf("overflow at %f, maximum value is %f", maxval, thisval );
	if ( maxval < thisval )
	    printf("Error, this set of input parameters will overflow");

	xn = 12357	/* seed value */
	printf("               Iteration                          Number");
	printf("                                                  DecimalHex");
	for( i=0 ; i < 50 ; i++ )
	{
	    xnp1 = ( a * xn + c ) % m;
	    printf("                 %6d                               %ld%lx",i,xnp1,xnp1);
	    xn = xnp1;
	}
	printf("");
	printf("                                              Table %1d",tnum);
	printf("              First 50 numbers of Linear Congruential Method");
	printf("                              With M=%ld, A=%ld, C=%ld",m,a,c);
}
/* end of file */
