/****************************************************************/
/*   NOISE GENERATION ROUTINES USING THE			*/
/*   WE(R) DSP16 DIGITAL SIGNAL PROCESSOR			*/
/*   November, 1988 Doc# AP88-18DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1988 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*								*/
/*			random1.s				*/
/*								*/
/*   The following is the program segment for the first DSP	*/
/*   example. This program segment generates the linear		*/
/*   congruential method of random number generation, i.e.	*/
/*								*/
/*      x(n+1) = (a*x(n) + c) mod n				*/
/*								*/
/* Inputs:  RAM variables  xnl, xnh - seed value		*/
/*          ROM locations  a, c, m  - parameters for algorithm	*/
/* Outputs: RAM variables  xnl, xnh - next random number	*/
/*          RAM variable udev - normalized 16 bit random number */
/*          a0 - random number					*/
/*          a1 - 16 bit normilized random number		*/

	/* declaration of ram variables */
.ram
xnl:	int	/* low 16 bits of seed number/previously	*/
		/* generated number.  xn gets overwritten with	*/
		/* xnp1 for use at the next call		*/
xnh:	int	/* high 16 bits of number			*/
tmpl:	int
tmph:	int
udev:	int	/* returned uniform deviate in ram		*/
.endram

udev0:
	pt=lcds					/* set rom ptr		*/
	r1=xnl					/* point to old number	*/
	a0=y					/* just to clr a0	*/
	a0=a0-y		y=*r1++	x=*pt++		/* y=xnl, x=a		*/
	a1=y					/* a1=xnl		*/
		p=x*y	y=*r1			/* p=lo protd, y=xnh	*/
  if pl goto cont0				/* if xnl pl, continue	*/
	a0=x					/* load x to a0		*/
cont0:
	a0=a0+p	p=x*y				/* a=unsign mul,p=hi prod */
test0:	a1=p					/* a1=hi prod, a0=lo prod */
	a1=a1<<16				/* a1=hi prod in MSW	  */
	*r1=a1
			y=*r1--			/* to get to upper bits	*/
	a0=a0+y					/* now have a*xn	*/
			y=*r1	x=*pt++		/* y=<>, x=c		*/
			y=0x1
		p=x*y
	a0=a0+p					/* a0 = a*x+c		*/

   /* a0 now has sum right justified  */
	r1=tmpl
			y=*r1	x=*pt++		/* y=<>, x=ml		*/
			*r1++=x			/* tmpl = ml		*/
			y=*r1	x=*pt++
			*r1=x
			y=*r1--			/* transfer to y	*/
			yl=*r1

/* a0 has the sum from which the modulus will be taken.  y has the modulus */
/* shifted to the maximum magnitude. The number of shifts depends upon the */
/* modulus. The modulus 134,456 shifted 13 times results in 0x41a70000 =   */
/* 1,101,463,552. This is the maximum shifted value less than 2^31-1. The  */
/* do loop will calculate the modulus as follows:			   */
/*     if a0 > y, subtract y						   */
/*     shift y right ( >> ) by one					   */
/*     repeat until y is the true modulus value				   */
/* The number of iterations is one plus the number of shifts in obtaining  */
/* the maximum modulus.							   */

  do 14 {
	a1=a0-y					/* test for magnitude	*/
	if pl a0=a1				/* subtract		*/
cont1:	a1=y
	a1=a1>>1				/* shift down		*/
	*r1++=a1				/* store high half	*/
	a1=a1<<16				/* get LSBs		*/
	*r1--=a1				/* store low half	*/
	y=*r1++					/* get high half in y	*/
	yl=*r1--				/* get low half		*/
	}

  /* a0 now has modularized value, store it into xnp1h, xnp1l */
	a1=a0					/* for use later	*/
	r1=xnh
	*r1--=a0
	a0=a0<<16
	*r1=a0

  /* now scale the number for +/- 2^15; scale factor (1.949)     */
  /* depends on ratio of value of modulus to the next power of 2 */
scl1:	a1=a1<<8				/* get to MSW		*/
	a1=a1<<4
			y=*r1	x=*pt++		/* x=scale factor	*/
			y=a1			/* y=number		*/
		p=x*y				/* scale to 16 bits	*/
	a0=p
	a0=a0<<4
	r1=udev
	*r1=a0					/* store in XRET	*/
udev0e: return

lcds:
	int  8121	/* A: multiplier			*/
	int  28411	/* C: increment				*/
	int  0x0000	/* ML: low half of shifted modulus	*/
	int  0x41a7	/* MH: high half of shifted modulus	*/
	int  1.949	/* scale				*/

/* end of file */
