/****************************************************************/
/*   NOISE GENERATION ROUTINES USING THE			*/
/*   WE(R) DSP16 DIGITAL SIGNAL PROCESSOR			*/
/*   November, 1988 Doc# AP88-18DMOS				*/
/****************************************************************/
/*	 Copyright (c) 1988 AT&T -- All Rights Reserved		*/
/*   WE is a registered trademark of AT&T. AT&T reserves the	*/
/*   right to make changes to  the software described herein	*/
/*   without notice.  No liability is assumed as a result of	*/
/*   their  use or  application.  No rights under any patent	*/
/*   accompany the sale of product.				*/
/****************************************************************/
/*								*/
/*			random2.s				*/
/*								*/
/*   The following is the program segment for the second DSP	*/
/*   example. This program segment generates the linear		*/
/*   congruential method of random number generation, i.e.	*/
/*								*/
/*      x(n+1) = (a*x(n) + c) mod m				*/
/*								*/
/*   The program is optimized for modulus of power of two.	*/
/*   Assumed modulus is 2^20.					*/
/*								*/
/* Inputs:  RAM variables  XNL, XNH - seed value		*/
/*          ROM locations  A, C  - parameters for algorithm	*/
/* Outputs: RAM variables  XNL, XNH - next 20 bit random number	*/
/*          RAM variable udev - normalized 16 bit random number */
/*          a0 - 20 bit random number				*/
/*          a1 - 16 bit normilized random number		*/

	/* declaration of ram variables */
.ram
xnl:	int	/* low 16 bits of seed number/previously	*/
		/* generated number.  xn gets overwritten with	*/
		/* xnp1 for use at the next call		*/
xnh:	int	/* high 16 bits of number			*/
udev:	int	/* returned uniform deviate in ram		*/
.endram

udev1:
	pt=lcds					/* set rom ptr		*/
	r1=xnl					/* point to old number	*/
	a0=y					/* just to clr a0	*/
	a0=a0-y		y=*r1++	x=*pt++		/* y=xnl, x=a		*/
	a1=y					/* a1=xnl		*/
		p=x*y	y=*r1			/* p=lo protd, y=xnh	*/
  if pl goto cont				/* if xnl pl, continue	*/
	a0=x					/* load x to a0		*/
cont:
	a0=a0+p	p=x*y				/* a=unsign mul,p=hi prod */
test1:	a1=p					/* a1=hi prod, a0=lo prod */
	a1=a1<<16				/* a1=hi prod in MSW	  */
	*r1=a1
			y=*r1--			/* to get to upper bits	*/
	a0=a0+y					/* now have a*xn	*/
			y=*r1	x=*pt++		/* y=<>, x=c		*/
			y=0x1
		p=x*y
	a0=a0+p					/* a0 = a*x+c		*/

   /* a0 now has sum right justified  */
	y=0xf					/* gen. 20 bit mask in y */
	yl=0xffff				/* LSW of mask		*/
	a0=a0&y					/* mask = modulus	*/

  /* a0 now has modular value, store it into xnp1h, xnp1l */
	a1=a0					/* save for later	*/
	*r1++=a1l				/* r1=xnl		*/
	*r1++=a1				/* r1=xnl		*/

  /* move to sign plus 15-bits of precision, and store	*/
  /* we assume that the multiplier is 2^20		*/
	a0=a0<<8				/* 2^28			*/
	a0=a0<<4				/* 2^32			*/
	*r1=a0					/* store in RAM		*/
udev1e: return

lcds:
	int  2045	/* A: multiplier			*/
	int  1		/* C: increment				*/
	int  0		/* M: modulus is hardwired in code	*/

/* end of file */
